import cml
from cml import load_knowledge

from sklearn.cluster import KMeans
from sklearn.cluster import SpectralClustering
from sklearn.feature_selection import VarianceThreshold
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.cluster import AgglomerativeClustering, DBSCAN
from sklearn.svm import SVC


def demo():

    source = cml.get_source("/home/dmt/.cml/data/double_toyset.csv")
    kmeans = KMeans(precompute_distances=False, algorithm="elkan", n_jobs=2)
    spectral = SpectralClustering(affinity='rbf', n_jobs=2)
    algorithms = [("Dbs", DBSCAN()),
                  ("Agg", AgglomerativeClustering())]
    constructor = cml.construction("conceptual", algorithms)
    constructor.min_category_size = 1

    variance = VarianceThreshold(2000)
    forest = ExtraTreesClassifier(n_estimators=250, random_state=1)
    selector = cml.feature_selection(filter_method=variance,
                                     embedded_method=forest)

    svc = SVC(gamma="auto", C=1)
    tree = DecisionTreeClassifier(max_depth=5)
    algos = [("Svc", svc), ("Tre", tree)]
    reconstructor = cml.reconstruction("conceptual", algos)
    deconstructor = cml.deconstruction("conceptual", source)

    result = cml.search_knowledge(constructor,
                                  selector,
                                  reconstructor,
                                  deconstructor,
                                  stdout=True,
                                  parallel=False)

    print("Finished")
    result.dump("/tmp/lol")
    knowledge = load_knowledge("/tmp/lol")

    print(knowledge.database[1].biggest_id)
    source = cml.get_source("/home/dmt/.cml/data/double_toyset.csv")
    deconstructor = cml.deconstruction("conceptual", source, knowledge=knowledge)
    print("Starting Next Knowledge search")
    input()
    result_2 = cml.search_knowledge(constructor,
                                    selector,
                                    reconstructor,
                                    deconstructor,
                                    stdout=True,
                                    parallel=False)

    source = cml.get_source("/home/dmt/.cml/data/double_toyset.csv")
    deconstructor = cml.deconstruction("conceptual", source, knowledge=knowledge)
    print("Starting Next Knowledge search")
    input()
    result_2 = cml.search_knowledge(constructor,
                                    selector,
                                    reconstructor,
                                    deconstructor,
                                    stdout=True,
                                    parallel=False)

if __name__ == '__main__':
    demo()
